/** @file
 * @brief provides tracing funtionality.
 *
 * Provides a scope tracker 
 *
 * @copyright &copy; Copyright 2008 by Intra2net AG
 * @license commercial
 * 
 * info@intra2net.com
 */

#ifndef __I2N_COMMON_TRACEFUNC_HPP__
#define __I2N_COMMON_TRACEFUNC_HPP__

#include <string>
#include "logfunc.hpp"
#include "source_track_basics.hpp"


namespace I2n
{
namespace Tracer
{


/**
 * @brief scope tracker class.
 *
 * basically: emits a ENTER message on construction and a LEAVE message on destruction.
 * And indent these messages acoording to the nesting depth.
 */
class ScopeTracker
{
   public:
     ScopeTracker(const SourceLocation& loc);
     ~ScopeTracker();

   private:
     ScopeTracker(const ScopeTracker& rhs);
     ScopeTracker& operator = (const ScopeTracker& rhs);

     SourceLocation Location;
     int Depth;
     int FuncDepth;

     std::string Tag;
}; // eo ScopeTracker


/*
** helper macros:
*/

/**
 * @brief constructs a scope tracker.
 */
#define SCOPETRACKER() ::I2n::Tracer::ScopeTracker __scope_tracker(HERE)


} // eo namespace Tracer
} // eo namespace I2n

#endif
